@@ -1,16 +1,25 @@
-package App::ClusterSSH::Cluster;
-
 use strict;
 use warnings;
 
-use version;
-our $VERSION = version->new('0.01');
+package App::ClusterSSH::Cluster;
+$App::ClusterSSH::Cluster::VERSION = '4.13.2_01'; # TRIAL
+
+$App::ClusterSSH::Cluster::VERSION = '4.13.201';# ABSTRACT: App::ClusterSSH::Cluster - Object representing cluster configuration
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+Object representing application configuration
+
+=cut
 
 use Carp;
-use Try::Tiny;
+use Try::Tiny 0.28;
 use English qw( -no_match_vars );
 
 use base qw/ App::ClusterSSH::Base /;
+use App::ClusterSSH::Range;
 
 our $master_object_ref;
 
@@ -51,9 +60,8 @@ sub get_tag_entries {
 sub list_external_clusters {
     my ( $self, ) = @_;
 
-    my @list =  $self->_run_external_clusters('-L');
-    return
-        wantarray
+    my @list = $self->_run_external_clusters('-L');
+    return wantarray
         ? sort @list
         : scalar @list;
 }
@@ -69,8 +77,12 @@ sub _run_external_clusters {
 
     my $external_command = $self->parent->config->{external_cluster_command};
 
-    if(!$external_command || ! -x $external_command) {
-        $self->debug( 1, 'Cannot run external cluster command: ', $external_command || '');
+    if ( !$external_command || !-x $external_command ) {
+        $self->debug(
+            1,
+            'Cannot run external cluster command: ',
+            $external_command || ''
+        );
         return;
     }
 
@@ -111,8 +123,37 @@ sub _run_external_clusters {
     return @results;
 }
 
+sub expand_filename {
+    my ( $self, $filename ) = @_;
+    my $home;
+
+    # try to determine the home directory
+    if ( !defined( $home = $ENV{'HOME'} ) ) {
+        $home = ( getpwuid($>) )[5];
+    }
+    if ( !defined($home) ) {
+        $self->debug( 3, 'No home found so leaving filename ',
+            $filename, ' unexpanded' );
+        return $filename;
+    }
+    $self->debug( 4, 'Using ', $home, ' as home directory' );
+
+    # expand ~ or $HOME
+    my $new_name = $filename;
+    $new_name =~ s!^~/!$home/!g;
+    $new_name =~ s!^\$HOME/!$home/!g;
+
+    $self->debug( 2, 'Expanding ', $filename, ' to ', $new_name )
+        unless ( $filename eq $new_name );
+
+    return $new_name;
+}
+
 sub read_tag_file {
     my ( $self, $filename ) = @_;
+
+    $filename = $self->expand_filename($filename);
+
     $self->debug( 2, 'Reading tags from file ', $filename );
     if ( -f $filename ) {
         my %hosts
@@ -130,6 +171,9 @@ sub read_tag_file {
 
 sub read_cluster_file {
     my ( $self, $filename ) = @_;
+
+    $filename = $self->expand_filename($filename);
+
     $self->debug( 2, 'Reading clusters from file ', $filename );
 
     if ( -f $filename ) {
@@ -150,6 +194,8 @@ sub register_host {
     my ( $self, $node, @tags ) = @_;
     $self->debug( 2, "Registering node $node on tags:", join( ' ', @tags ) );
 
+    @tags = $self->expand_glob( 'node', $node, @tags );
+
     foreach my $tag (@tags) {
         if ( $self->{tags}->{$tag} ) {
             $self->{tags}->{$tag}
@@ -167,6 +213,11 @@ sub register_host {
 sub register_tag {
     my ( $self, $tag, @nodes ) = @_;
 
+    #warn "b4 nodes=@nodes";
+    @nodes = $self->expand_glob( 'tag', $tag, @nodes );
+
+    #warn "af nodes=@nodes";
+
     $self->debug( 2, "Registering tag $tag: ", join( ' ', @nodes ) );
 
     $self->{tags}->{$tag} = \@nodes;
@@ -174,6 +225,35 @@ sub register_tag {
     return $self;
 }
 
+sub expand_glob {
+    my ( $self, $type, $name, @items ) = @_;
+
+    my @expanded;
+    my $range = App::ClusterSSH::Range->new();
+
+    # skip expanding anything that appears to have nasty metachars
+    if ( !grep {m/[\`\!\$;]/} @items ) {
+
+        $self->debug( 4, "Non-expanded: @items" );
+
+        @items = $range->expand(@items);
+
+     # run glob over anything left incase there are numeric and textual ranges
+        @expanded = map { glob $_ } @items;
+        $self->debug( 4, "Final expansion: @expanded" );
+    }
+    else {
+        warn(
+            $self->loc(
+                "Bad characters picked up in [_1] '[_2]': [_3]",
+                $type, $name, join( ' ', @items )
+            ),
+        );
+    }
+
+    return @expanded;
+}
+
 sub get_tag {
     my ( $self, $tag ) = @_;
 
@@ -184,8 +264,7 @@ sub get_tag {
             join( ' ', sort @{ $self->{tags}->{$tag} } )
         );
 
-        return
-            wantarray
+        return wantarray
             ? sort @{ $self->{tags}->{$tag} }
             : scalar @{ $self->{tags}->{$tag} };
     }
@@ -196,8 +275,7 @@ sub get_tag {
 
 sub list_tags {
     my ($self) = @_;
-    return
-        wantarray
+    return wantarray
         ? sort keys( %{ $self->{tags} } )
         : scalar keys( %{ $self->{tags} } );
 }
@@ -217,18 +295,6 @@ sub dump_tags {
 
 1;
 
-=pod
-
-=head1 NAME
-
-App::ClusterSSH::Cluster - Object representing cluster configuration
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-Object representing application configuration
-
 =head1 METHODS
 
 =over 4
@@ -259,6 +325,10 @@ file name and register the tags found.
 
 Read in the given cluster file and register the tags found
 
+=item $cluster->expand_filename($filename);
+
+Expand ~ or $HOME in a filename
+
 =item $cluster->read_tag_file($filename);
 
 Read in the given tag file and register the tags found
@@ -286,22 +356,8 @@ Return an array of all available tag names
 
 Returns a hash of all tag data.
 
+=item @tags = $cluster->expand_glob( $type, $name, @items );
+
+Use shell expansion against each item in @items, where $type is either 'node', or 'tag' and $name is the node or tag name.  These attributes are presented to the user in the event of an issue with the expanion to track down the source.
+
 =back
-
-=head1 AUTHOR
-
-Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 1999-2010 Duncan Ferguson.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
-
-=cut
-
-1;
