@@ -1,10 +1,18 @@
-package App::ClusterSSH::Config;
-
 use strict;
 use warnings;
 
-use version;
-our $VERSION = version->new('0.02');
+package App::ClusterSSH::Config;
+$App::ClusterSSH::Config::VERSION = '4.13.2_01'; # TRIAL
+
+$App::ClusterSSH::Config::VERSION = '4.13.201';# ABSTRACT: ClusterSSH::Config - Object representing application configuration
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+Object representing application configuration
+
+=cut
 
 use Carp;
 use Try::Tiny;
@@ -77,6 +85,8 @@ my %default_config = (
     extra_tag_file           => '',
     extra_cluster_file       => '',
     external_cluster_command => '',
+    external_command_mode    => '0600',
+    external_command_pipe    => '',
 
     unmap_on_redraw => "no",    # Debian #329440
 
@@ -524,18 +534,6 @@ sub dump {
 
 1;
 
-=pod
-
-=head1 NAME
-
-ClusterSSH::Config - Object representing application configuration
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-Object representing application configuration
-
 =head1 METHODS
 
 =over 4
@@ -576,21 +574,3 @@ are loaded).
 Write currently defined configuration to STDOUT
 
 =back
-
-=head1 AUTHOR
-
-Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 1999-2016 Duncan Ferguson.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
-
-=cut
-
-1;
