@@ -1,15 +1,23 @@
-package App::ClusterSSH::Getopt;
-
 use strict;
 use warnings;
 
-use version;
-our $VERSION = version->new('0.01');
+package App::ClusterSSH::Getopt;
+$App::ClusterSSH::Getopt::VERSION = '4.13.2_01'; # TRIAL
+
+$App::ClusterSSH::Getopt::VERSION = '4.13.201';# ABSTRACT: App::ClusterSSH::Getopt - module to process command line args
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+Object representing application configuration
+
+=cut
 
 use Carp;
 use Try::Tiny;
 use Pod::Usage;
-use Getopt::Long qw(:config no_ignore_case bundling no_auto_abbrev);
+use Getopt::Long 2.5 qw(:config no_ignore_case bundling no_auto_abbrev);
 use FindBin qw($Script);
 
 use base qw/ App::ClusterSSH::Base /;
@@ -18,7 +26,7 @@ sub new {
     my ( $class, %args ) = @_;
 
     # basic setup that is over-rideable by each script as needs may be
-    # different depending ont he command used
+    # different depending on the command used
     my %setup = (
         usage => [
             '-h|--help', '[options] [[user@]<server>[:port]|<tag>] [...] ',
@@ -46,19 +54,28 @@ sub add_option {
     }
     my ( $option, $arg ) = $spec =~ m/^(.*?)(?:[\+=:](.*))?$/;
     if ($arg) {
-        my $arg_type = defined $args{arg_desc} ? "<$args{arg_desc}>" : undef;
+        my $arg_open  = '<';
+        my $arg_close = '>';
+        if ( $args{arg_optional} ) {
+            $arg_open  = '[';
+            $arg_close = ']';
+        }
+        my $arg_type
+            = defined $args{arg_desc}
+            ? "${arg_open}$args{arg_desc}${arg_close}"
+            : undef;
         $arg =~ s/\+/[[...] || <INTEGER>]/g;
         if ( $arg eq 'i' ) {
             $arg
                 = defined $arg_type
                 ? $arg_type
-                : q{<} . $self->loc('INTEGER') . q{>};
+                : $arg_open . $self->loc('INTEGER') . $arg_close;
         }
         if ( $arg eq 's' ) {
             $arg
                 = defined $arg_type
                 ? "'$arg_type'"
-                : q{'<} . $self->loc('STRING') . q{>'};
+                : "'" . $arg_open . $self->loc('STRING') . $arg_close . "'";
         }
     }
     my ( $desc, $long, $short, $accessor );
@@ -117,7 +134,7 @@ sub add_common_options {
     $self->add_option(
         spec => 'debug:+',
         help => $self->loc(
-            "Enable debugging.  Either a level can be provided or the option can be repeated multiple times.  Maximum level is 4."
+            "Enable debugging.  Either a level can be provided or the option can be repeated multiple times.  Maximum level is 9."
         ),
         default => 0,
     );
@@ -175,8 +192,12 @@ sub add_common_options {
         ),
     );
     $self->add_option(
-        spec => 'list|L',
-        help => $self->loc('List available cluster tags.'),
+        spec => 'list|L:s',
+        help => $self->loc(
+            'List available cluster tags. Tag is optional.  If a tag is provided then hosts for that tag are listed.  NOTE: format of output changes when using "--quiet" or "-Q" option.'
+        ),
+        arg_desc     => 'tag',
+        arg_optional => 1,
     );
     $self->add_option(
         spec => 'dump-config|d',
@@ -222,6 +243,27 @@ sub add_common_options {
             'If a hostname resolves to multiple IP addresses, toggle whether or not to connect to all of them, or just the first one (see also config file entry).'
         ),
     );
+    $self->add_option(
+        spec => 'quiet|Q',
+        help =>
+            $self->loc('Do not output extra text when using some options'),
+    );
+    $self->add_option(
+        spec     => 'cols|x=i',
+        arg_desc => 'cols',
+        help     => $self->loc('Number of columns'),
+    );
+    $self->add_option(
+        spec     => 'rows|y=i',
+        arg_desc => 'rows',
+        help     => $self->loc('Number of rows'),
+    );
+
+    $self->add_option(
+        spec => 'fillscreen',
+        help => $self->loc(
+            'Resize terminal windows to fill the whole available screen'),
+    );
 
     return $self;
 }
@@ -288,10 +330,14 @@ sub getopts {
     }
 
     $options->{debug} ||= 0;
-    $options->{debug} = 4 if ( $options->{debug} && $options->{debug} > 4 );
+    $options->{debug} = 9 if ( $options->{debug} && $options->{debug} > 9 );
 
     # Now all options are set to the correct values, generate accessor methods
     foreach my $option ( sort keys( %{ $self->{command_options} } ) ) {
+
+        # skip some accessors as they are already defined elsewhere
+        next if $option =~ m/^(debug)\W/;
+
         my $accessor = $self->{command_options}->{$option}->{accessor};
 
         my $default = $self->{command_options}->{$option}->{default};
@@ -303,7 +349,7 @@ sub getopts {
           # hide warnings when getopts is run multiple times, esp. for testing
             no warnings 'redefine';
             *$accessor = sub {
-                return $options->{$acc} || $default;
+                return defined $options->{$acc} ? $options->{$acc} : $default;
 
     #                      defined $options->{$acc} ? $options->{$acc}
     #                    : defined $self->{command_options}->{$acc}->{default}
@@ -315,7 +361,7 @@ sub getopts {
         }
     }
 
-    $self->set_debug_level( $self->debug );
+    $self->set_debug_level( $options->{debug} );
 
     $self->parent->config->load_configs( $self->config_file );
 
@@ -358,6 +404,14 @@ sub getopts {
             = !$self->parent->config->{window_tiling} || 0;
     }
 
+    if ( $self->rows ) {
+        $self->parent->config->{rows} = $self->rows;
+    }
+    if ( $self->cols ) {
+        $self->parent->config->{cols} = $self->cols;
+    }
+    $self->parent->config->{fillscreen} = "yes"
+        if ( $self->fillscreen );
     return $self;
 }
 
@@ -376,7 +430,10 @@ sub _generate_pod {
     output $/ , "=pod";
     output '=head1 ',    $self->loc('NAME');
     output "$Script - ", $self->loc("Cluster administration tool");
-    output '=head1 ',    $self->loc('SYNOPSIS');
+    output '=head1 ',    $self->loc('VERSION');
+    output $self->loc( "This documentation is for version: [_1]",
+        $self->parent->VERSION );
+    output '=head1 ', $self->loc('SYNOPSIS');
 
     # build the synopsis
     print "$Script ";
@@ -495,7 +552,7 @@ would replace the <Alt-n> with the client's name in each window.}
     );
     output '=item ', $self->parent->config->{key_localname};
     output $self->loc(
-        q{Paste in the hostname of the server cssh is ebing run on});
+        q{Paste in the hostname of the server cssh is being run on});
     output '=item ', $self->parent->config->{key_quit};
     output $self->loc(
         'Quit the program and close all connections and windows.');
@@ -518,13 +575,14 @@ would replace the <Alt-n> with the client's name in each window.}
         $self->loc(
         q{Connect to different servers using different login names.  NOTE: this can also be achieved by setting up appropriate options in the configuration files.  Do not close the console when the last terminal exits.}
         );
-    output q{S<$ } . $Script . q{ -Q user1@server1 admin@server2>};
+    output q{S<$ } . $Script . q{ user1@server1 admin@server2>};
     output '=item ',
         $self->loc(
         q{Open up a cluster defined in a non-default configuration file});
     output q{S<$ } . $Script . q{ -c $HOME/cssh.extra_clusters db_cluster>};
     output '=item ',
-        $self->loc(q{Connect on port 2022 instead of the default port});
+        $self->loc(
+        q{Override the configured/default port to use 2022 instead});
     output q{S<$ } . $Script . q{ -p 2022 server1 server2>};
     output '=back';
 
@@ -544,11 +602,20 @@ would replace the <Alt-n> with the client's name in each window.}
     output $self->loc(
         q{All comments (marked by a #) and blank lines are ignored.  Tags may be nested, but be aware of using recursive tags as they are not checked for.}
     );
+    output $self->loc(q{Servers can be defined using expansion macros:});
+    output 'C<< webservers websvr{a,b,c} >>';
+    output $self->loc(q{would be expanded to});
+    output 'C<< webservers websvra websvrb websvrc >>';
+    output $self->loc(q{and});
+    output 'C<< webservers websvr{6..9} >>';
+    output $self->loc(q{would be expanded to});
+    output 'C<< webservers websvr6 websvr7 websvr8 websvr9 >>';
+
     output $self->loc(
         q{Extra cluster files may also be specified either as an option on the command line (see [_1]) or in the user's [_2] file (see [_3] configuration option).},
         'C<cluster-file>',
         'F<$HOME/.clusterssh/config>',
-        'C<extra_cluster_file>'
+        'L</extra_cluster_file>'
     );
     output $self->loc(
         'B<NOTE:> the last tag read overwrites any pre-existing tag of that name.'
@@ -601,6 +668,11 @@ would replace the <Alt-n> with the client's name in each window.}
         'L<--autoquit>',
     );
 
+    output '=item auto_wm_decoration_offsets = no';
+    output $self->loc(
+        'Enable or disable alternative algorithm for calculating terminal positioning.',
+    );
+
     output '=item comms = ' . $self->parent->config->{comms};
     output $self->loc(
         'Sets the default communication method (initially taken from the name of the program, but can be overridden here).'
@@ -611,6 +683,22 @@ would replace the <Alt-n> with the client's name in each window.}
         q{Set the initial position of the console - if empty then let the window manager decide.  Format is '+<x>+<y>', i.e. '+0+0' is top left hand corner of the screen, '+0-70' is bottom left hand side of screen (more or less).}
     );
 
+    output '=item external_command_mode = 0600';
+    output $self->loc(q{File mode bits for the external_command_pipe.});
+
+    output '=item external_command_pipe = <null>';
+    output $self->loc(
+        q{Define the full path to an external command pipe that can be written to for controlling some aspects of ClusterSSH, such as opening sessions to more clusters.
+
+Commands:
+
+C<< open <tag|hostname> >> - open new sessions to provided tag or hostname
+
+C<< retile >> - force window retiling
+    
+e.g.: C<< echo 'open localhost' >> /path/to/external_command_pipe >>}
+    );
+
     output '=item external_cluster_command = <null>';
     output $self->loc(
         q{Define the full path to an external command that can be used to resolve tags to host names.  This command can be written in any language.  The script must accept a list of tags to resolve and output a list of hosts (space separated on a single line).  Any tags that cannot be resolved should be returned unchanged.
@@ -627,12 +715,24 @@ If the external command is given a C<-L> option it should output a list of tags
     );
     output 'C<< extra_cluster_file = ~/clusters, $HOME/clus >>';
 
+    output '=item extra_tag_file = <null>';
+    output $self->loc(
+        q{Define an extra tag file in the format of [_1].  Multiple files can be specified, separated by commas.  Both ~ and $HOME are acceptable as a reference to the user's home directory, e.g.},
+        'F</etc/tags>'
+    );
+    output 'C<< extra_tag_file = ~/tags, $HOME/tags >>';
+
     output '=item key_addhost = Control-Shift-plus';
     output $self->loc(
         q{Default key sequence to open AddHost menu.  See [_1] for more information.},
         'L<KEY SHORTCUTS>'
     );
 
+    output '=item hide_menu = 0';
+    output $self->loc(
+        q{If set to 1, hide the menu bar (File, Hosts, Send, Help) in the console.},
+    );
+
     output '=item key_clientname = Alt-n';
     output $self->loc(
         q{Default key sequence to send cssh client names to client.  See [_1] for more information.},
@@ -743,6 +843,18 @@ B<NOTE:> Any "generic" change to the method (e.g., specifying the ssh port to us
         'F<$HOME/.Xdefaults>', 'F<$HOME/.Xresources>'
     );
 
+    output '=item terminal_chdir = 0';
+    output $self->loc(
+        q{When non-0, set the working directory for each terminal as per '[_1]'},
+        'L<terminal_chdir_path>'
+    );
+
+    output '=item terminal_chdir_path = $HOME/.clusterssh/work/%s';
+    output $self->loc(
+        q{Path to use as working directory for each terminal when '[_1]' is enabled.  The path provided is passed through the macro parser (see the section above on '[_2]'.},
+        'L<terminal_chdir>', 'L<macros_enabled>',
+    );
+
     output '=item terminal_font = 6x13';
     output $self->loc(
         q{Font to use in the terminal windows.  Use standard X font notation.}
@@ -801,6 +913,12 @@ B<NOTE:> Any "generic" change to the method (e.g., specifying the ssh port to us
     output '=item use_hotkeys = 1';
     output $self->loc( q{Setting to [_1] will disable all hotkeys.}, 'C<0>' );
 
+    output '=item use_natural_sort = 0';
+    output $self->loc(
+        q{Windows will normally sort in alphabetical order, i.e.: host1, host11, host2.  Setting to this [_1] will change the sort order, i.e.: host1, host2, host11. NOTE: You must have the perl module [_2] installed.},
+        'C<1>', 'L<Sort::Naturally>'
+    );
+
     output '=item user = $LOGNAME';
     output $self->loc(
         q{Sets the default user for running commands on clients.});
@@ -822,7 +940,7 @@ B<NOTE:> Any "generic" change to the method (e.g., specifying the ssh port to us
         'C<Control>', 'C<Alt>', 'C<Shift>', 'C<null>'
     );
 
-    output q{=item F<$HOME/.csshrc_send_menu>};
+    output q{=item F<$HOME/.clusterssh/send_menu>};
     output $self->loc(
         q{This (optional) file contains items to populate the send menu.  The default entry could be written as:}
     );
@@ -904,9 +1022,8 @@ B<NOTE:> Any "generic" change to the method (e.g., specifying the ssh port to us
     );
     output '=back';
 
-    output '=head1 ', $self->loc('REPORTING BUGS');
-    output '=over';
-    output '=item *';
+    output '=head1 ', $self->loc('TROUBLESHOOTING');
+
     output $self->loc(
         q{If you have issues running [_1], first try:
 
@@ -927,16 +1044,16 @@ This performs two tests to confirm cssh is able to work properly with the settin
         $self->parent->config->{comms} );
     output '=back';
 
-    output $self->loc(q{Configuration options to watch for in ssh are});
+    output $self->loc(q{Configuration options to watch for in ssh are:});
     output '=over';
-    output '=item ',
-        $self->loc(
+    output '=item *';
+    output $self->loc(
         q{SSH doesn't understand [_1] - remove the option from the [_2] file},
         'C<-o ConnectTimeout=10>',
         'F<$HOME/.clusterssh/config>'
-        );
-    output '=item ',
-        $self->loc(
+    );
+    output '=item *';
+    output $self->loc(
         q{OpenSSH-3.8 using untrusted ssh tunnels - use [_1] instead of [_2] or use [_3] in [_4] (if you change the default ssh options from [_5] to [_6])},
         'C<-Y>',
         'C<-X>',
@@ -944,12 +1061,19 @@ This performs two tests to confirm cssh is able to work properly with the settin
         'F<$HOME/.ssh/ssh_config>',
         'C<-x>',
         'C<-X>'
-        );
+    );
     output '=back';
 
-    output '=item *';
+    output '=head1 ', $self->loc('SUPPORT AND REPORTING BUGS');
+
     output $self->loc(
-        q{If you require support, please run the following commands and post it on the web site in the support/problems forum:}
+        q{A web site for comments, requests, bug reports and bug fixes/patches is available at: [_1]},
+        'L<https://github.com/duncs/clusterssh>'
+    );
+
+    output $self->loc(
+        q{If you require support, please run the following commands and create an issue via: [_1]},
+        'L<https://github.com/duncs/clusterssh/issues>',
     );
     output 'C<< perl -V >>';
     output q{C<< perl -MTk -e 'print $Tk::VERSION,$/' >>};
@@ -957,35 +1081,27 @@ This performs two tests to confirm cssh is able to work properly with the settin
         q{C<< perl -MX11::Protocol -e 'print $X11::Protocol::VERSION,$/' >>};
     output 'C<< cat /etc/csshrc $HOME/.clusterssh/config >>';
 
-    output '=item *';
     output $self->loc(
         q{Using the debug option (--debug) will turn on debugging output.  Repeat the option to increase the amount of debug.  However, if possible please only use this option with one host at a time, e.g. [_1] due to the amount of output produced (in both main and child windows).},
         'C<< cssh --debug <host> >>'
     );
-    output '=back';
 
     output '=head1 ', $self->loc('SEE ALSO');
     output $self->loc(
-        q{L<http://clusterssh.sourceforge.net/>,
+        q{L<https://github.com/duncs/clusterssh/wiki/>,
 C<ssh>,
 L<Tk::overview>,
 L<X11::Protocol>,
 C<perl>}
     );
 
-    output '=head1 ', $self->loc('CREDITS');
-    output $self->loc(
-        'A web site for comments, requests, bug reports and bug fixes/patches is available at: [_1]',
-        'L<https://github.com/duncs/clusterssh>'
-    );
-
     output '=head1 ', $self->loc('AUTHOR');
     output 'Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>';
 
     output '=head1 ', $self->loc('LICENSE AND COPYRIGHT');
     output $self->loc(
         q{
-Copyright 1999-2014 Duncan Ferguson.
+Copyright 1999-2018 Duncan Ferguson.
 
 This program is free software; you can redistribute it and/or modify it under the terms of either: the GNU General Public License as published by the Free Software Foundation; or the Artistic License.
 
@@ -998,20 +1114,6 @@ See http://dev.perl.org/licenses/ for more information.
 
 1;
 
-__DATA__
-
-=pod
-
-=head1 NAME
-
-App::ClusterSSH::Getopt - module to process command line args
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-Object representing application configuration
-
 =head1 METHODS
 
 =over 4
@@ -1055,21 +1157,3 @@ Simple helper func to print out pod lines with double returns
 Functions to output help and usage instructions
 
 =back
-
-=head1 AUTHOR
-
-Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 1999-2010 Duncan Ferguson.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
-
-=cut
-
-1;
