@@ -1,10 +1,18 @@
-package App::ClusterSSH::Helper;
-
 use strict;
 use warnings;
 
-use version;
-our $VERSION = version->new('0.02');
+package App::ClusterSSH::Helper;
+$App::ClusterSSH::Helper::VERSION = '4.13.2_01'; # TRIAL
+
+$App::ClusterSSH::Helper::VERSION = '4.13.201';# ABSTRACT: ClusterSSH::Helper - Object representing helper script
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+Object representing application configuration
+
+=cut
 
 use Carp;
 use Try::Tiny;
@@ -22,7 +30,10 @@ sub new {
 sub script {
     my ( $self, $config ) = @_;
 
-    if(! defined $config || ! ref $config || ref $config ne "App::ClusterSSH::Config") {
+    if (   !defined $config
+        || !ref $config
+        || ref $config ne "App::ClusterSSH::Config" )
+    {
         croak(
             App::ClusterSSH::Exception::Helper->throw(
                 error => 'No configuration provided or in wrong format',
@@ -30,9 +41,10 @@ sub script {
         );
     }
 
-    foreach my $arg ( "comms", $config->{comms}, $config->{comms} . '_args', 'command', 'auto_close'
-    ) {
-        if( !defined $config->{ $arg } ) {
+    foreach my $arg ( "comms", $config->{comms}, $config->{comms} . '_args',
+        'command', 'auto_close' )
+    {
+        if ( !defined $config->{$arg} ) {
             croak(
                 App::ClusterSSH::Exception::Helper->throw(
                     error => "Config '$arg' not provided",
@@ -95,7 +107,8 @@ sub script {
             \$command .= " \\\"$config_command\\\"";
            }
            \$command .= " ; $postcommand";
-           warn("Running:\$command\\n"); # for debug purposes
+           # provide some info for debugging purposes
+           warn("Running: \$command\\n");
            exec(\$command);
     HERE
 
@@ -115,18 +128,6 @@ sub script {
 
 1;
 
-=pod
-
-=head1 NAME
-
-ClusterSSH::Helper - Object representing helper script
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-Object representing application configuration
-
 =head1 METHODS
 
 =over 4
@@ -140,21 +141,3 @@ Create a new helper object.
 Return the helper script
 
 =back
-
-=head1 AUTHOR
-
-Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 1999-2010 Duncan Ferguson.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
-
-=cut
-
-1;
