@@ -1,10 +1,26 @@
-package App::ClusterSSH::Host;
-
 use strict;
 use warnings;
 
-use version;
-our $VERSION = version->new('0.03');
+package App::ClusterSSH::Host;
+$App::ClusterSSH::Host::VERSION = '4.13.2_01'; # TRIAL
+
+$App::ClusterSSH::Host::VERSION = '4.13.201';# ABSTRACT: ClusterSSH::Host - Object representing a host.
+
+=head1 SYNOPSIS
+
+    use ClusterSSH::Host;
+
+    my $host = ClusterSSH::Host->new({
+        hostname => 'hostname',
+    });
+    my $host = ClusterSSH::Host->parse_host_string('username@hostname:1234');
+
+=head1 DESCRIPTION
+
+Object representing a host.  Include details to contact the host such as
+hostname/ipaddress, username and port.
+
+=cut
 
 use Carp;
 use Net::hostent;
@@ -309,9 +325,11 @@ sub check_ssh_hostname {
         $self->get_hostname );
 
     if ( $ssh_hostname_for{ $self->get_hostname } ) {
+        $self->debug( 5, 'Found' );
         return 1;
     }
     else {
+        $self->debug( 5, 'Not found' );
         return 0;
     }
 }
@@ -326,26 +344,6 @@ use overload (
 
 1;
 
-=pod
-
-=head1 NAME
-
-ClusterSSH::Host - Object representing a host.
-
-=head1 SYNOPSIS
-
-    use ClusterSSH::Host;
-
-    my $host = ClusterSSH::Host->new({
-        hostname => 'hostname',
-    });
-    my $host = ClusterSSH::Host->parse_host_string('username@hostname:1234');
-
-=head1 DESCRIPTION
-
-Object representing a host.  Include details to contact the host such as
-hostname/ipaddress, username and port.
-
 =head1 METHODS
 
 =over 4
@@ -429,21 +427,3 @@ the IPv6 address or a port definition?) and assumes it is part of address.
 Use brackets to avoid seeing warning.
 
 =back
-
-=head1 AUTHOR
-
-Duncan Ferguson, C<< <duncan_j_ferguson at yahoo.co.uk> >>
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 1999-2016 Duncan Ferguson.
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
-
-=cut
-
-1;
